/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/wx.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "incs/constants.h"
#include "incs/Configuration.h"

/**
 * Constructor
 */
Configuration::Configuration()
{
  FILE *fp;

  homeDir = (char*) malloc(255);
  configFile = (char*) malloc(255);
  hostFile = (char*) malloc(255);
  bzero(homeDir, 255);
  bzero(configFile, 255);
  bzero(hostFile, 255);

  sprintf(homeDir, "%s/%s", getenv("HOME"), GISKARDDIR);
  sprintf(configFile, "%s/%s", homeDir, CONFIGFILE);
  sprintf(hostFile, "%s/%s", homeDir, HOSTSFILE);
  fp = fopen(hostFile, "r");
  if(fp != NULL)
    fclose(fp);
  else
  {
#ifdef DEBUG
    printf("Creating Giskard directory (%s)\n", homeDir);
#endif
    if(mkdir(homeDir, S_IRWXU) == 0 || errno == EEXIST)
    {
#ifdef DEBUG
      printf("Creating default Host-file (%s)\n", hostFile);
#endif
      fp = fopen(hostFile, "w");
      fprintf(fp, "localhost");
      fclose(fp);
    }
    else
    {
      perror("FATAL ERROR ON CREATING DEFAULT CONFIGURATION");
    }
  }
}

/**
 * Destructor
 */
Configuration::~Configuration()
{
  free(hostFile);
  free(configFile);
  free(homeDir);
}

/**
 * Gibt die Directory zurück, wo alle Hilfsdateien gespeichert werden
 *
 * @return char*            die Directory mit den Hilfsdateien
 */
char* Configuration::getHomeDir()
{
  return this->homeDir;
}

/**
 * Gibt die Hauptkonfigurationsdatei zurück
 *
 * @return char*            der Pfad zur Datei
 */
char* Configuration::getConfigFile()
{
  return this->configFile;
}

/**
 * Gibt die Datei mit den Hosts zurück
 *
 * @return char*            der Pfad zur Datei
 */
char* Configuration::getHostFile()
{
  return this->hostFile;
}

/**
 * Gibt die Liste der möglichen Hosts (zum Login) zurück
 *
 * @param wxString* hosts   die Hosts als Referenz
 * @param int*      nHosts  der Anzahl der Hosts
 * @return bool             true wenn alles in Ordnung
 */
bool Configuration::getHostList(wxString *hosts, int *nHosts)
{
  FILE *fp;
  char host[255], *p;
  wxString hostStr;
  int n;

  n = 0;
#ifdef DEBUG
  printf("Reading Hosts (%s)\n", this->hostFile);
#endif
  fp = fopen(this->hostFile, "r");
  if(fp != NULL)
  {
    while(!feof(fp))
    {
      if(fgets(host, 255, fp) != NULL)
      {
        if((p = strchr(host, '\n' )))
          *p = '\0';
#ifdef DEBUG
        printf("host: [%s]\n", host);
#endif
        hostStr = wxString::FromUTF8(host);
        hosts[n++] = hostStr;
      }
    }
    fclose(fp);
    *nHosts = n;

    return true;
  }

  return false;
}

/**
 * Gibt den Pfad zur Hilfsseite zurück
 *
 * @return wxString         der Pfad zur Datei
 */
wxString Configuration::getHelpMainPage()
{
  wxString main, lang, dir, file;

  dir = wxT(HELPDIR);
  file = wxT(MAINHELPFILE);
  lang = wxString::FromUTF8(getenv("LANG"));
  lang = lang.Left(2);
  main.Printf(wxT("%s/%s/%s"), dir.c_str(), lang.c_str(), file.c_str());

  return main;
}
