/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/wx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <jpeglib.h>
#include "incs/constants.h"
#include "incs/GiskardConnector.h"

/**
 * Constructor
 */
GiskardConnector::GiskardConnector()
{
  this->connected = false;
  this->tryed = false;

  bzero(this->cameraTmpFile, 255);
  strcpy(this->cameraTmpFile, tmpnam(NULL));
  bzero(this->flatTmpFile, 255);
  strcpy(this->flatTmpFile, tmpnam(NULL));
}

/**
 * Destructor
 */
GiskardConnector::~GiskardConnector()
{
}

/**
 * Gibt den Host zurück, an dem das Programm verbunden ist
 *
 * @return char*                    der Host
 */
char *GiskardConnector::getHost()
{
  return this->host;
}

/**
 * Gibt den Pfad zur Datei zurück, wo das aktuelle Bild aus der
 * Kamera gespeichert ist
 *
 * @return wxString                 der Pfad zur Datei
 */
wxString GiskardConnector::getImageFromCamera()
{
  struct addrinfo addrinfo;
  struct addrinfo *res, *r;
  SOCKET sockCamera;
  FILE *fd, *fp;
  char garbage1[80], garbage2[30], *stream;
  int jpglength;

  bzero(garbage1, 80);
  bzero(garbage2, 30);

  memset(&addrinfo, 0, sizeof(addrinfo));
  addrinfo.ai_family=PF_UNSPEC;
  addrinfo.ai_socktype=SOCK_STREAM;
  addrinfo.ai_protocol=IPPROTO_TCP;
  if(getaddrinfo(this->host, CAMERAPORT, &addrinfo, &res) != 0)
    return wxT("");
  for(r = res; r; r = r->ai_next)
  {
    sockCamera = socket(r->ai_family, r->ai_socktype, r->ai_protocol);
    if(connect(sockCamera, r->ai_addr, r->ai_addrlen) >= 0)
    {
      fd = fdopen(sockCamera, "r");

      while(strncmp(garbage1, "Content-Length:", 15) != 0)
        fgets(garbage1, 70, fd);

      sscanf(garbage1, "%s%d", garbage2, &jpglength);
      fgets(garbage1, 70, fd);

      stream = (char*) malloc(sizeof(char) * (jpglength + 2));
      bzero(stream, jpglength);
      fread(stream, 1, jpglength, fd);

      fp = fopen(this->cameraTmpFile, "w");
      fwrite(stream, 1, jpglength, fp);
      fflush(fp);
      fclose(fp);

      fclose(fd);
      free(stream);
      close(sockCamera);

      return wxString(this->cameraTmpFile, wxConvUTF8);
    }
  }
  freeaddrinfo(res);

  return wxT("");
}

/**
 * Gibt den Pfad zur Datei zurück, wo das Bild mit dem Grundriss
 * der Wohnung gespeichert ist
 *
 * @return wxString                 der Pfad zur Datei
 */
wxString GiskardConnector::getFlatImage()
{
  char str[1024];
  int n;
  long imageSize, i;
  FILE *fp;

  send(this->sock, "GETFLAT\n", strlen("GETFLAT\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "250 ", 4) == 0)
    {
      sscanf(str, "250 DATA (%ld bytes)", &imageSize);
      fp = fopen(this->flatTmpFile, "wb");
      for(i = 0; i < imageSize;)
      {
        n = recv(this->sock, str, 1024, 0);
        if(n > 0)
        {
          i += n;
          fwrite(str, 1, n, fp);
        }
        else
          i = imageSize + 1;
      }
      fflush(fp);
      fclose(fp);
      return wxString(this->flatTmpFile, wxConvUTF8);
    }
  }

  return wxT("ERROR");
}

/**
 * Gibt die Position des Roboters innerhalb der Wohnung zurück
 *
 * @return wxPoint                  die Koordinaten
 */
wxPoint GiskardConnector::getRobotPosition()
{
  wxPoint position;
  char str[1024];
  int n;

  if(getenv("FAKE") != NULL)
  {
    position.x = 100;
    position.y = 50;
    return position;
  }

  send(this->sock, "GETPOSITION\n", strlen("GETPOSITION\n"), 0);

  bzero(str, 1024);
  position.x = 0;
  position.y = 0;
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
      sscanf(str, "200 %d;%d", &position.x, &position.y);
  }

  return position;
}

/**
 * Öffnet ein Socket zum Roboter
 *
 * @return bool                     true wenn erfolgreich verbunden, false ansonsten (der Fehler wird angezeigt)
 */
bool GiskardConnector::connectSocket()
{
  struct addrinfo addrinfo;
  struct addrinfo *res, *r;
  wxString error;

  memset(&addrinfo, 0, sizeof(addrinfo));
  addrinfo.ai_family=PF_UNSPEC;
  addrinfo.ai_socktype=SOCK_STREAM;
  addrinfo.ai_protocol=IPPROTO_TCP;
  if(getaddrinfo(this->host, GISKARDPORT, &addrinfo, &res) != 0)
  {
    error.Printf(_("Es war nicht möglich '%s' zu übersetzten"), this->host);
    strcpy(this->error, error.mb_str(wxConvUTF8));
    return false;
  }
  for(r = res; r; r = r->ai_next)
  {
    this->sock = socket(r->ai_family, r->ai_socktype, r->ai_protocol);
    if(connect(this->sock, r->ai_addr, r->ai_addrlen) >= 0)
      return true;
    error.Printf(_("Keine Verbindung mit '%s'"), this->host);
    strcpy(this->error, error.mb_str(wxConvUTF8));
    return false;
  }
  freeaddrinfo(res);

  error.Printf(_("Allgemeiner Fehler"));
  strcpy(this->error, error.mb_str(wxConvUTF8));
  return false;
}

/**
 * Verbindet mit dem Roboter
 *
 * @param char* login               der Nutzername
 * @param char* passwd              das Passwort
 * @param char* host                die Adresse des Roboters
 * @return bool                     true wenn erfolgreich verbunden, false ansonsten (der Fehler wird angezeigt)
 */
bool GiskardConnector::connectToRobot(char *login, char *passwd, char *host)
{
  char str[1024];
  int n, ready;
  wxString error;

  strcpy(this->login, login);
  strcpy(this->passwd, passwd);
  strcpy(this->host, host);

  this->tryed = true;

  if(this->connectSocket())
  {
    ready = 0;
    while(ready == 0)
    {
      bzero(str, 1024);
      n = recv(this->sock, str, 255, 0);
      if(n > 0)
      {
        if(strncmp(str, "220 READY", 9) == 0)
          ready = 1;
        if(strncmp(str, "550 ", 4) == 0)
          ready = 2;
      }
    }

    if(ready == 1)
    {
      bzero(str, 1024);
      sprintf(str, "USER:%s\n", this->login);
      send(this->sock, str, strlen(str), 0);
      n = recv(this->sock, str, 255, 0);
      bzero(str, 1024);
      sprintf(str, "PASS:%s\n", this->passwd);
      send(this->sock, str, strlen(str), 0);

      bzero(str, 1024);
      n = recv(this->sock, str, 255, 0);
      if(n > 0)
      {
        if(strncmp(str, "250 ", 4) == 0)
        {
          this->connected = true;
          return true;
        }
        error.Printf(_("Ungültiges Login oder Passwort"));
        strcpy(this->error, error.mb_str(wxConvUTF8));
      }
    }
    else
    {
      error.Printf(_("Der Roboter ist mit einem anderen Host verbunden"));
      strcpy(this->error, error.mb_str(wxConvUTF8));
    }
  }
  this->connected = false;
  closesocket(this->sock);
  return false;
}

/**
 * Gibt die Loginfehlermeldung zurück
 *
 * @return wxString                 die Fehlermeldung
 */
wxString GiskardConnector::getLoginError()
{
  return wxString(this->error, wxConvUTF8);
}

/**
 * Gibt zurück, ob einen Verbindungsversuch stattgefunden hat
 *
 * @return bool                     true wenn hat versucht
 */
bool GiskardConnector::hasTryed()
{
  return this->tryed;
}

/**
 * Setzt zurück die Verbindungsversuche
 */
void GiskardConnector::discardTry()
{
  this->tryed = false;
}

/**
 * Gibt zurück, ob das Programm mit dem Roboter verbunden ist
 *
 * @return bool                     true wenn verbunden
 */
bool GiskardConnector::isConnected()
{
  return this->connected;
}

/**
 * Trennt die Verbindung mit dem Roboter.
 * Schickt ein "LOGOUT" zum Programm und löscht die Dateien mit den Daten
 * der Kamera und des Grundrisses
 *
 * @return bool                     true wenn erfolgreich
 */
bool GiskardConnector::disconnectFromRobot()
{
  char str[1024];
  int n;

  send(this->sock, "LOGOUT\n", strlen("LOGOUT\n"), 0);
  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "250 ", 4) != 0)
      return false;
  }
  else
    return false;

  closesocket(this->sock);

  unlink(this->cameraTmpFile);
  unlink(this->flatTmpFile);

  this->connected = false;
  this->tryed = false;
  printf("DISCONNECT\n");

  return true;
}

/**
 * Gibt zurück den Wert der Batterie des Mikrocontrollers
 *
 * @return wxString                 der Wert der Batterie als Prozent
 */
wxString GiskardConnector::getUCBattery()
{
  char str[1024];
  int n;
  wxString ret;
  double val;

  if(getenv("FAKE") != NULL)
    return wxT("65.4%");

  send(this->sock, "GETUCBATT\n", strlen("GETUCBATT\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
    {
      ret = wxString(str, wxConvUTF8).After(' ');
      ret.ToDouble(&val);
      val /= 100;
      return wxString::Format(wxT("%3.1f%%"), val);
    }
  }

  return wxT("ERROR");
}

/**
 * Gibt zurück den Wert der Batterie des Laptops
 *
 * @return wxString                 der Wert der Batterie als Prozent
 */
wxString GiskardConnector::getBattery()
{
  char str[1024];
  int n;
  wxString ret;
  double val;

  if(getenv("FAKE") != NULL)
    return wxT("98.7%");

  send(this->sock, "GETBATT\n", strlen("GETBATT\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
    {
      ret = wxString(str, wxConvUTF8).After(' ');
      ret.ToDouble(&val);
      return wxString::Format(wxT("%3.1f%%"), val);
    }
  }

  return wxT("ERROR");
}

/**
 * Gibt zurück den Wert des Kompasses
 *
 * @return wxString                 der Wert des Kompasses
 */
wxString GiskardConnector::getCompass()
{
  char str[1024];
  int n;
  wxString ret;
  double val;

  if(getenv("FAKE") != NULL)
    return wxT("123.4°");

  send(this->sock, "GETCOMPASS\n", strlen("GETCOMPASS\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
    {
      ret = wxString(str, wxConvUTF8).After(' ');
      ret.ToDouble(&val);
      val /= 10;
      return wxString::Format(wxT("%3.1f°"), val);
    }
  }

  return wxT("ERROR");
}

/**
 * Gibt zurück die Entfernung zum nächsten Objekt vor dem Roboter
 * oder "Freier Weg" wenn der Weg frei ist
 *
 * @return wxString                 die Entfernung
 */
wxString GiskardConnector::getObjects()
{
  char str[1024];
  int n;
  wxString ret;
  long int val;

  if(getenv("FAKE") != NULL)
    return _("Freier Weg");

  send(this->sock, "GETOBJECTS\n", strlen("GETOBJECTS\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
    {
      ret = wxString(str, wxConvUTF8).After(' ');
      ret.ToLong(&val);
      if(strncmp(str, "200 -1", 5) == 0)
        return _("Freier Weg");
      else
        return wxString::Format(wxT("%i cm"), val);
    }
  }

  return wxT("ERROR");
}

/**
 * Gibt zurück ob der FrontLED eingeschaltet ist
 *
 * @return bool                     true wenn eingeschaltet, false wenn ausgeschaltet
 */
bool GiskardConnector::getFrontLED()
{
  char str[1024], *p;
  int n;
  wxString ret;

  if(getenv("FAKE") != NULL)
    return false;

  send(this->sock, "GETFRONTLED\n", strlen("GETFRONTLED\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if((p = strchr(str, '\n')))
      *p = '\0';
    if(strncmp(str, "200 ", 4) == 0)
    {
      ret = wxString(str, wxConvUTF8).After(' ');
      if(ret == _T("ON"))
        return true;
      else
        return false;
    }
  }

  return false;
}

/**
 * Schaltet das FrontLED ein oder aus
 *
 * @param bool value                true wenn das LED einzuschalten ist, false wenn auszuschalten
 * @return bool                     true wenn erfolgreich ein-, bzw. ausgeschaltet
 */
bool GiskardConnector::setFrontLED(bool value)
{
  char str[1024];
  int n;

  if(getenv("FAKE") != NULL)
    return true;

  if(value)
    sprintf(str, "SETFRONTLED ON\n");
  else
    sprintf(str, "SETFRONTLED OFF\n");
  send(this->sock, str, strlen(str), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
      return true;
    else
      return false;
  }

  return false;
}

/**
 * Gibt die Richtung der Kamera (Servo) zurück
 *
 * @return int                      die Richtung der Kamera, zwischen 5 und 250
 */
int GiskardConnector::getCameraDirection()
{
  char str[1024];
  int n;
  wxString ret;
  long int val;

  if(getenv("FAKE") != NULL)
    return 123;

  send(this->sock, "GETCAMERAPOSITION\n", strlen("GETCAMERAPOSITION\n"), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
    {
      ret = wxString(str, wxConvUTF8).After(' ');
      ret.ToLong(&val);
      return val;
    }
  }

  return -1;
}

/**
 * Bewegt die Kamera (Servo)
 *
 * @param int value                 die Richtung (zwischen 5 und 250)
 * @return bool                     true wenn bewegt
 */
bool GiskardConnector::setCameraDirection(int value)
{
  char str[1024];
  int n;

  if(getenv("FAKE") != NULL)
    return true;

  sprintf(str, "SETCAMERAPOSITION %d\n", value);
  send(this->sock, str, strlen(str), 0);

  bzero(str, 1024);
  n = recv(this->sock, str, 255, 0);
  if(n > 0)
  {
    if(strncmp(str, "200 ", 4) == 0)
      return true;
    else
      return false;
  }

  return false;
}
