#include <wx/wx.h>
#include "incs/constants.h"
#include "incs/GiskardConnector.h"
#include "incs/GiskardRemoteGUIMainFrame.h"
#include "incs/GiskardRemoteGUILoginDialog.h"
#include "incs/GiskardRemoteGUILoginWaitDialog.h"
#include "incs/GiskardRemoteGUILoginError.h"

GiskardRemoteGUILoginDialog::GiskardRemoteGUILoginDialog( wxWindow* parent )
:
GenGiskardRemoteGUILoginDialog( parent )
{
  int l;

  this->myParent = (GiskardRemoteGUIMainFrame*) parent;
  if(this->myParent->config->getHostList(this->hosts, &this->nHosts))
  {
    for(l = 0; l < this->nHosts; l++)
      hostInput->Append(this->hosts[l]);
  }
  loginInput->SetFocus();
  m_sdbSizer1OK->Disable();
}

/**
 * Schließt das Fenster
 */
void GiskardRemoteGUILoginDialog::doClose( wxCloseEvent& event )
{
  this->Show(false);
}

/**
 * Aktiviert, bzw. deaktiviert das "OK"-Knopf.
 * Es kann ein Login erfolgen nur wenn Nutzername, Passwort und Host angegeben wurden
 */
void GiskardRemoteGUILoginDialog::checkDataForLogin( wxCommandEvent& event )
{
  if(loginInput->GetValue().IsEmpty() == false &&
     passwdInput->GetValue().IsEmpty() == false &&
     hostInput->GetSelection() != wxNOT_FOUND)
    m_sdbSizer1OK->Enable();
  else
    m_sdbSizer1OK->Disable();
}

/**
 * Der Nutzer hat auf "Cancel" geklickt
 */
void GiskardRemoteGUILoginDialog::doExit( wxCommandEvent& event )
{
  this->Destroy();
}

/**
 * Versucht sich mit dem Roboter über TCP/IP zu verbinden.
 * Es wird einen "Warte"-Fenster angezeigt (GiskardRemoteGUILoginWaitDialog).
 * Im Falle von Fehler wird die Fehlermeldung angezeigt (GiskardRemoteGUILoginError)
 */
void GiskardRemoteGUILoginDialog::doLogin( wxCommandEvent& event )
{
  GiskardRemoteGUILoginWaitDialog *loginWait = new GiskardRemoteGUILoginWaitDialog(this);
  strcpy(loginWait->loginStr, loginInput->GetValue().mb_str());
  strcpy(loginWait->passwdStr, passwdInput->GetValue().mb_str());
  strcpy(loginWait->hostStr, this->hosts[hostInput->GetSelection()].mb_str());

  loginWait->giskardConnector = this->myParent->giskardConnector;
  if(loginWait->ShowModal() == CONNECT_ERROR)
  {
    this->toDisplayError = this->myParent->giskardConnector->getLoginError();
    GiskardRemoteGUILoginError *loginError = new GiskardRemoteGUILoginError(this);
    loginError->ShowModal();
    loginWait->giskardConnector->discardTry();
  }
  else
  {
    this->Show(false);
    this->myParent->afterConnect();
  }
}
