#include <wx/wx.h>
#include <wx/sstream.h>
#include <time.h>
#include "resources/giskard.xpm"
#include "incs/constants.h"
#include "incs/GiskardConnector.h"
#include "incs/GiskardRemoteGUIMainFrame.h"
#include "incs/GiskardRemoteGUILoginDialog.h"
#include "incs/GiskardRemoteGUIAboutDialog.h"
#include "incs/GiskardRemoteGUIHelpDialog.h"

/**
 * Constructor
 * Initialisiert der GiskardConnector, setzt das Programmicon, gibt ein
 * Standardwert für alle Objekte
 */
GiskardRemoteGUIMainFrame::GiskardRemoteGUIMainFrame( wxWindow* parent )
:
GenGiskardRemoteGUIMainFrame( parent )
{
  wxBitmap bmp;
  wxImage img;
  wxSize size;

  SetIcon(wxICON(giskard));
  this->giskardConnector = new GiskardConnector();
  this->m_toolBar1->EnableTool(wxID_DISCONNECT, false);
  this->mFrontLEDOn->Enable(false);
  this->mFrontLEDOff->Enable(false);
  this->mFrontLEDOn->SetValue(false);
  this->mFrontLEDOff->SetValue(true);
  this->frontLED = false;
  this->mCameraDir->Enable(false);

  size = this->mCameraBMP->GetSize();
  img = wxImage(size.GetWidth(), size.GetHeight());
  bmp = wxBitmap(img);
  this->mCameraBMP->SetBitmap(bmp);

  size = this->mFlatBMP->GetSize();
  img = wxImage(size.GetWidth(), size.GetHeight());
  bmp = wxBitmap(img);
  this->mFlatBMP->SetBitmap(bmp);
}

/**
 * Zeigt das Logindialog und erlaubt die Verbindung
 */
void GiskardRemoteGUIMainFrame::doConnect( wxCommandEvent& event )
{
  GiskardRemoteGUILoginDialog *loginDlg = new GiskardRemoteGUILoginDialog(this);
  loginDlg->ShowModal();
}

/**
 * Trennt die Verbindung mit dem Roboter
 */
void GiskardRemoteGUIMainFrame::doDisconnect( wxCommandEvent& event )
{
  if(this->giskardConnector->disconnectFromRobot())
  {
    this->afterDisconnect();
  }
}

/**
 * Schließt das Programm
 */
void GiskardRemoteGUIMainFrame::doCloseProgram( wxCommandEvent& event )
{
  if(this->giskardConnector->isConnected())
    this->giskardConnector->disconnectFromRobot();
  exit(0);
}

/**
 * Zeigt die Hilfe zum Programm
 */
void GiskardRemoteGUIMainFrame::doHelp( wxCommandEvent& event )
{
  GiskardRemoteGUIHelpDialog *helpDlg = new GiskardRemoteGUIHelpDialog(this);
  helpDlg->ShowModal();
}

/**
 * Zeigt einige Informationen zum Programm (About)
 */
void GiskardRemoteGUIMainFrame::doShowAbout( wxCommandEvent& event )
{
  GiskardRemoteGUIAboutDialog *aboutDlg = new GiskardRemoteGUIAboutDialog(this);
  aboutDlg->ShowModal();
}

/**
 * Schaltet das FrontLED ein
 */
void GiskardRemoteGUIMainFrame::doTurnFrontLEDOn( wxCommandEvent& event )
{
  if(!this->frontLED)
  {
    if(!this->giskardConnector->setFrontLED(true))
    {
      this->mFrontLEDOn->SetValue(false);
      this->mFrontLEDOff->SetValue(true);
      this->frontLED = true;
    }
  }
}

/**
 * Schaltet das FrontLED aus
 */
void GiskardRemoteGUIMainFrame::doTurnFrontLEDOff( wxCommandEvent& event )
{
  if(this->frontLED)
  {
    if(!this->giskardConnector->setFrontLED(false))
    {
      this->mFrontLEDOn->SetValue(true);
      this->mFrontLEDOff->SetValue(false);
      this->frontLED = false;
    }
  }
}

/**
 * Zeigt den aktuellen Wert der Batterie des Laptops
 */
void GiskardRemoteGUIMainFrame::setBattery()
{
  wxColor color;
  double perc;

  this->mBatterie->SetLabel(this->giskardConnector->getBattery());
  this->giskardConnector->getBattery().ToDouble(&perc);
  if(perc > 80)
    color = wxColor(22, 254, 5);
  else if(perc > 40)
    color = wxColor(195, 138, 0);
  else
    color = wxColor(255, 0, 0);
  this->mBatterie->SetForegroundColour(color);
}

/**
 * Zeigt den aktuellen Wert der Hauptbatterie
 */
void GiskardRemoteGUIMainFrame::setUCBattery()
{
  wxColor color;
  double perc;

  this->mUCBatterie->SetLabel(this->giskardConnector->getUCBattery());
  this->giskardConnector->getUCBattery().ToDouble(&perc);
  if(perc > 50)
    color = wxColor(22, 254, 5);
  else if(perc > 20)
    color = wxColor(195, 138, 0);
  else
    color = wxColor(255, 0, 0);
  this->mUCBatterie->SetForegroundColour(color);
}

/**
 * Holt das Status des FrontLEDs vom Roboter
 */
void GiskardRemoteGUIMainFrame::setFrontLED()
{
  if(this->giskardConnector->getFrontLED())
  {
    this->mFrontLEDOn->SetValue(true);
    this->mFrontLEDOff->SetValue(false);
    this->frontLED = true;
  }
  else
  {
    this->mFrontLEDOn->SetValue(false);
    this->mFrontLEDOff->SetValue(true);
    this->frontLED = false;
  }
}

/**
 * Holt die Richtung der Kamera (Servo) vom Roboter
 */
void GiskardRemoteGUIMainFrame::setCameraDir()
{
  int val = this->giskardConnector->getCameraDirection();

  if(val != -1)
    this->mCameraDir->SetValue(val);
}

/**
 * Bewegt die Kamera (Servo)
 */
void GiskardRemoteGUIMainFrame::doMoveCamera( wxScrollEvent& event )
{
  this->giskardConnector->setCameraDirection(this->mCameraDir->GetValue());
}

/**
 * Zeigt das aktuelle Bild aus der Kamera
 */
void GiskardRemoteGUIMainFrame::drawCamera()
{
  wxBitmap bmp;
  wxImage img;
  wxSize size;

  img = wxImage(this->giskardConnector->getImageFromCamera(), wxBITMAP_TYPE_JPEG);
  size = this->mCameraBMP->GetSize();
  bmp = wxBitmap(img.Rescale(size.GetWidth(), size.GetHeight()));
  this->mCameraBMP->SetBitmap(bmp);
}

/**
 * Zeigt den Grundriss der Wohnung
 */
void GiskardRemoteGUIMainFrame::drawFlat()
{
  wxSize size;
  wxImage img;
  wxPoint point;

  img = wxImage(this->giskardConnector->getFlatImage(), wxBITMAP_TYPE_GIF);
  size = this->mFlatBMP->GetSize();
  this->flatBMP = wxBitmap(img.Rescale(size.GetWidth(), size.GetHeight()));
  this->mFlatBMP->SetBitmap(this->flatBMP);
}

/**
 * Zeigt die aktuelle Position des Roboters
 */
void GiskardRemoteGUIMainFrame::drawRobotPosition()
{
  wxCoord radius = 5;
  wxPoint point, flatPos, robotPos;
  wxPaintDC dc(this->mFlatBMP);
  wxColour col;
  wxBrush brush;

  col.Set(wxT("#ff0000"));
  brush.SetColour(col);
  brush.SetStyle(wxSOLID);
  dc.SetBrush(brush);
  flatPos = this->mFlatBMP->GetPosition();
  robotPos = this->giskardConnector->getRobotPosition();
  point.x = flatPos.x + robotPos.x;
  point.y = flatPos.y + robotPos.y;

  dc.DrawBitmap(this->flatBMP, flatPos.x, flatPos.y, false);
  dc.DrawCircle(point, radius);
}

/**
 * Aufgerufen nach einer erfolgreichen Verbidung mit dem Roboter.
 * Holt alle Werte und zeigt sie.
 * Zeigt das Bild aus der Kamera und die Position des Roboters
 */
void GiskardRemoteGUIMainFrame::afterConnect()
{
  wxString timeLabel, host;

  this->m_toolBar1->EnableTool(wxID_DISCONNECT, true);
  this->m_toolBar1->EnableTool(wxID_CONNECT, false);
  this->miConnect->Enable(false);
  this->miDisconnect->Enable(true);
  host = wxString(this->giskardConnector->getHost(), wxConvUTF8);
  this->setBattery();
  this->setUCBattery();
  this->setFrontLED();
  this->setCameraDir();
  this->mCompass->SetLabel(this->giskardConnector->getCompass());
  this->mObjects->SetLabel(this->giskardConnector->getObjects());
  this->m_timer.SetOwner(this);
  this->m_timer.Start(500); // Alle 0.5 Sekunde, wegen der Kamera
  this->connectionStart = time(NULL);
  this->lastQuery = this->connectionStart;
  timeLabel.Printf(_("Verbunden [%02d:%02d.%02d]"), 0, 0, 0);
  this->mStatusLabel->SetLabel(timeLabel);
  this->mFrontLEDOn->Enable(true);
  this->mFrontLEDOff->Enable(true);
  this->mst_FrontLED->Enable(true);
  this->mst_Camera->Enable(true);
  this->mCameraDir->Enable(true);

  this->drawCamera();
  this->drawFlat();
}

/**
 * Aufgerufen nach einer erfolgreichen Trennung der Verbidung mit dem Roboter.
 * Löscht alle Werte und die Bilder der Kamera und der Grundriss
 */
void GiskardRemoteGUIMainFrame::afterDisconnect()
{
  wxBitmap bmp;
  wxImage img;
  wxSize size;

  this->m_toolBar1->EnableTool(wxID_DISCONNECT, false);
  this->m_toolBar1->EnableTool(wxID_CONNECT, true);
  this->miConnect->Enable(true);
  this->miDisconnect->Enable(false);
  this->mUCBatterie->SetLabel(wxT("--"));
  this->mBatterie->SetLabel(wxT("--"));
  this->mCompass->SetLabel(wxT("--"));
  this->mObjects->SetLabel(wxT("--"));
  this->mBatterie->SetForegroundColour(wxColor(0, 0, 0));
  this->mUCBatterie->SetForegroundColour(wxColor(0, 0, 0));
  this->m_timer.Stop();
  this->mStatusLabel->SetLabel(_("Verbindung getrennt"));
  this->mFrontLEDOn->Enable(false);
  this->mFrontLEDOff->Enable(false);
  this->mst_FrontLED->Enable(false);
  this->mst_Camera->Enable(false);
  this->mCameraDir->Enable(false);

  size = this->mCameraBMP->GetSize();
  img = wxImage(size.GetWidth(), size.GetHeight());
  bmp = wxBitmap(img);
  this->mCameraBMP->SetBitmap(bmp);

  size = this->mFlatBMP->GetSize();
  img = wxImage(size.GetWidth(), size.GetHeight());
  bmp = wxBitmap(img);
  this->mFlatBMP->SetBitmap(bmp);
}

/**
 * Aufgerufen vom Timer alle 0.5 Sekunden.
 * Aktuelisiert alle Werte und das Bild aus der Kamera
 */
void GiskardRemoteGUIMainFrame::OnTimerEvent(wxTimerEvent &event)
{
  wxString timeLabel;
  struct tm *elapsed;
  time_t now = time(NULL);
  time_t seconds = now - this->connectionStart;

  elapsed = gmtime(&seconds);

  timeLabel.Printf(_("Verbunden [%02d:%02d.%02d]"), elapsed->tm_hour, elapsed->tm_min, elapsed->tm_sec);
  this->mStatusLabel->SetLabel(timeLabel);

  this->drawCamera();

  if((now - this->lastQuery) >= QUERYINTERVAL)
  {
    this->setBattery();
    this->setUCBattery();
    this->mCompass->SetLabel(this->giskardConnector->getCompass());
    this->mObjects->SetLabel(this->giskardConnector->getObjects());
    this->setFrontLED();
    this->setCameraDir();
    this->drawRobotPosition();
    this->lastQuery = now;
  }
}

BEGIN_EVENT_TABLE(GiskardRemoteGUIMainFrame, MainFrame)
  EVT_TIMER(-1, GiskardRemoteGUIMainFrame::OnTimerEvent)
END_EVENT_TABLE()
