/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef CONFIGURATION_H_
#define CONFIGURATION_H_

/**
 * Konfiguration des Programms
 */
class Configuration
{
public:
  /**
   * Constructor
   */
  Configuration();
  virtual
  /**
   * Destructor
   */
  ~Configuration();

  /**
   * Gibt die Directory zurück, wo alle Hilfsdateien gespeichert werden
   *
   * @return char*            die Directory mit den Hilfsdateien
   */
  char* getHomeDir();

  /**
   * Gibt die Hauptkonfigurationsdatei zurück
   *
   * @return char*            der Pfad zur Datei
   */
  char* getConfigFile();

  /**
   * Gibt die Datei mit den Hosts zurück
   *
   * @return char*            der Pfad zur Datei
   */
  char* getHostFile();

  /**
   * Gibt die Liste der möglichen Hosts (zum Login) zurück
   *
   * @param wxString* hosts   die Hosts als Referenz
   * @param int*      nHosts  der Anzahl der Hosts
   * @return bool             true wenn alles in Ordnung
   */
  bool getHostList(wxString *hosts, int *nHosts);

  /**
   * Gibt den Pfad zur Hilfsseite zurück
   *
   * @return wxString         der Pfad zur Datei
   */
  wxString getHelpMainPage(void);

protected:
  /**
   * Directory mit den Hilfsdateien
   */
  char *homeDir;

  /**
   * Hauptkonfigurationsdatei
   */
  char *configFile;

  /**
   * Hostsdatei
   */
  char *hostFile;
};

#endif /* CONFIGURATION_H_ */
