/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef GiskardRemoteGUIMainFrame_H_
#define GiskardRemoteGUIMainFrame_H_

#include "../gen/GiskardRemoteGUIMainFrame.h"
#include "../giskardRemoteGUI.h"

#include "GiskardConnector.h"
#include "Configuration.h"

/**
 * Klasse des Hauptfensters
 */
class GiskardRemoteGUIMainFrame : public GenGiskardRemoteGUIMainFrame
{
  DECLARE_EVENT_TABLE()

	protected:
	  /**
	   * Schaltet das FrontLED ein
	   */
    void doTurnFrontLEDOn( wxCommandEvent& event );

    /**
     * Schaltet das FrontLED aus
     */
    void doTurnFrontLEDOff( wxCommandEvent& event );

    /**
     * Zeigt den aktuellen Wert der Batterie des Laptops
     */
    void setBattery(void);

    /**
     * Zeigt den aktuellen Wert der Hauptbatterie
     */
    void setUCBattery(void);

    /**
     * Holt das Status des FrontLEDs vom Roboter
     */
    void setFrontLED(void);

    /**
     * Holt die Richtung der Kamera (Servo) vom Roboter
     */
    void setCameraDir(void);

    /**
     * Bewegt die Kamera (Servo)
     */
    void doMoveCamera( wxScrollEvent& event );

    /**
     * Zeigt das aktuelle Bild aus der Kamera
     */
    void drawCamera(void);

    /**
     * Zeigt den Grundriss der Wohnung
     */
    void drawFlat(void);

    /**
     * Zeigt die aktuelle Position des Roboters
     */
    void drawRobotPosition(void);

    /**
     * Zeigt das Logindialog und erlaubt die Verbindung
     */
    void doConnect( wxCommandEvent& event );

    /**
     * Trennt die Verbindung mit dem Roboter
     */
    void doDisconnect( wxCommandEvent& event );

    /**
     * Schließt das Programm
     */
    void doCloseProgram( wxCommandEvent& event );

    /**
     * Zeigt die Hilfe zum Programm
     */
    void doHelp( wxCommandEvent& event );

    /**
     * Zeigt einige Informationen zum Programm (About)
     */
    void doShowAbout( wxCommandEvent& event );
	private:
	  /**
	   * Timerhilfsvariabel
	   */
    wxTimer m_timer;
    time_t connectionStart, lastQuery;

    /**
     * Status des FrontLEDs
     */
    bool frontLED;

    /**
     * Bitmap mit dem Grundriss der Wohnung
     */
    wxBitmap flatBMP;
	public:
	  /**
	   * Constructor
	   * Initialisiert der GiskardConnector, setzt das Programmicon, gibt ein
	   * Standardwert für alle Objekte
	   */
		GiskardRemoteGUIMainFrame( wxWindow* parent );

		/**
		 * GiskardConnector
		 */
		GiskardConnector *giskardConnector;

		/**
		 * Konfiguration des Programms
		 */
		Configuration *config;

		/**
		 * Aufgerufen nach einer erfolgreichen Verbidung mit dem Roboter.
		 * Holt alle Werte und zeigt sie
     * Zeigt das Bild aus der Kamera und die Position des Roboters
		 */
		void afterConnect();

		/**
		 * Aufgerufen nach einer erfolgreichen Trennung der Verbidung mit dem Roboter.
		 * Löscht alle Werte und die Bilder der Kamera und der Grundriss
		 */
		void afterDisconnect();

		/**
		 * Aufgerufen vom Timer alle 0.5 Sekunden.
		 * Aktuelisiert alle Werte und das Bild aus der Kamera
		 */
		void OnTimerEvent(wxTimerEvent &event);
};

#endif
