/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/wx.h>
#include "incs/constants.h"
#include "incs/Configuration.h"
#include "incs/GiskardRemoteGUIMainFrame.h"
#include "incs/GiskardRemoteGUILoginDialog.h"
#include "giskardRemoteGUI.h"

/**
 * Hauptklasse des Programms
 */
class GiskardRemote: public wxApp
{
  /**
   * Initialisierung
   */
  virtual bool OnInit();

  /**
   * Programm beenden
   */
  virtual int OnExit();

  /**
   * Konfiguration des Programms
   */
  Configuration *config;

  /**
   * Locales (für die Übersetzungen)
   */
  wxLocale m_locale;
};

IMPLEMENT_APP(GiskardRemote)

/**
 * Initialisierung
 */
bool GiskardRemote::OnInit()
{
  this->config = new Configuration();

  // Alle ImageHandlers laden (auch für JPG...)
  wxInitAllImageHandlers();

  // Nutze die Systemsprache (Variabel $LANG)
  this->m_locale.Init(wxLocale::GetSystemLanguage(), wxLOCALE_CONV_ENCODING);
  wxLocale::AddCatalogLookupPathPrefix(wxT(LOCALEDIR));
  this->m_locale.AddCatalog(wxT("giskard"));

  GiskardRemoteGUIMainFrame *mainFrame = new GiskardRemoteGUIMainFrame(NULL);
  mainFrame->config = this->config;
  mainFrame->Show(true);
  SetTopWindow(mainFrame);

  return true;
}

/**
 * Programm beenden
 */
int GiskardRemote::OnExit()
{
  delete(this->config);

  return 0;
}
